@@ -92,7 +92,7 @@ import sys
 import time
 import unittest
 from xml.sax import saxutils
-import sys
+import copy
 
 
 # ------------------------------------------------------------------------
@@ -540,7 +540,7 @@ class _TestResult(TestResult):
     # note: _TestResult is a pure representation of results.
     # It lacks the output and reporting ability compares to unittest._TextTestResult.
 
-    def __init__(self, verbosity=1):
+    def __init__(self, verbosity=1, trynum=0):
         TestResult.__init__(self)
         self.stdout0 = None
         self.stderr0 = None
@@ -562,6 +562,11 @@ class _TestResult(TestResult):
         # 增加一个测试通过率 --Findyou
         self.passrate = float(0)
 
+        # 增加用例失败重试
+        self.trynum = trynum
+        self.trys = 0
+        self.istry = False
+
     def startTest(self, test):
         TestResult.startTest(self, test)
         # just one buffer for both stdout and stderr
@@ -589,9 +594,29 @@ class _TestResult(TestResult):
         # Usually one of addSuccess, addError or addFailure would have been called.
         # But there are some path in unittest that would bypass this.
         # We must disconnect stdout in stopTest(), which is guaranteed to be called.
+        # 判断是否需要重试
+        if self.istry is True:
+            # 执行次数小于重试次数，就重试
+            if self.trys < self.trynum:
+                # 删除最后一个结果
+                result = self.result.pop(-1)
+                # 判断结果，如果是错误就把错误的个数减掉
+                # 如果是失败，就把失败的次数减掉
+                if result[0] == 1:
+                    self.failure_count -= 1
+                else:
+                    self.error_count -= 1
+                sys.stderr.write(f"{test.id()}:用例正在重试中..." + '\n')
+                test = copy.copy(test)
+                self.trys += 1
+                test(self)
+            else:
+                self.istry = False
+                self.trys = 0
         self.complete_output()
 
     def addSuccess(self, test):
+        self.istry=False
         self.success_count += 1
         TestResult.addSuccess(self, test)
         output = self.complete_output()
@@ -604,6 +629,7 @@ class _TestResult(TestResult):
             sys.stderr.write('.')
 
     def addError(self, test, err):
+        self.istry = True
         self.error_count += 1
         TestResult.addError(self, test, err)
         _, _exc_str = self.errors[-1]
@@ -630,6 +656,7 @@ class _TestResult(TestResult):
             sys.stderr.write('S')
 
     def addFailure(self, test, err):
+        self.istry = True
         self.failure_count += 1
         TestResult.addFailure(self, test, err)
         _, _exc_str = self.failures[-1]
@@ -647,9 +674,10 @@ class HTMLTestRunner(Template_mixin):
     """
     """
 
-    def __init__(self, stream=sys.stdout, verbosity=1, title=None, description=None, tester=None):
+    def __init__(self, stream=sys.stdout, verbosity=1, title=None, description=None, tester=None, trynum=0):
         self.stream = stream
         self.verbosity = verbosity
+        self.trynum = trynum
         if title is None:
             self.title = self.DEFAULT_TITLE
         else:
@@ -667,7 +695,7 @@ class HTMLTestRunner(Template_mixin):
 
     def run(self, test):
         "Run the given test case or test suite."
-        result = _TestResult(self.verbosity)
+        result = _TestResult(self.verbosity, self.trynum)
         test(result)
         self.stopTime = datetime.datetime.now()
         self.generateReport(test, result)
